/* Functors used as callbacks by the event-handler.
   Copyright (C) 2006 Carl Fredrik Hammar (hammy.lite@gmail.com)

   This file is part of GP2X Sand.

   GP2X Sand is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   GP2X Sand is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details. 

   You should have received a copy of the GNU General Public License along
   with GP2X Sand; if not, write to the Free Software Foundation, Inc., 51
   Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */

#ifndef ACTION_H
#define ACTION_H

#include "SDL.h"

/* This module is centered around actions, simple functors that handle
   certain types of events.  And also contains some convenient utilities
   that are useful for creating actions.  As actions are used to handle
   events they are used in conjunction with the eventhandler module. */

/* The type of the function that is part of an action and gets called when
   an action is invoked.  ACTION_PTR is a pointer to the action that got
   invoked and EVENT_PTR is a pointer to the event that triggered the
   invokation.  action_ptr is typed as a void pointer because it's typical
   to cast it to a containing structure which holds additional data, if no
   data is needed then action_ptr is usually not examined anyway.
   ACTION_PTR isn't null when the function is called through action_invoke
   but it doesn't hurt to test this whithin the function.. */ 
typedef void (*action_function_t) (void *action_ptr,
				   SDL_Event *event_ptr);

/* Actions couple functions with data to enable functions of diverse types
   to be wrapped and called through a uniform interface. */
struct action_str 
{
  action_function_t function;
};

typedef struct action_str action_t;
  
/* Initialize the action pointed to by ACTION_PTR to use FUNCTION.
   ACTION_PTR mustn't be null.  If FUNCTION is null this action does
   nothing on invocation. */
void
action_init (action_t *action_ptr, action_function_t function)
  __attribute__ ((nonnull (1)));

/* Invokes the action ACTION_PTR points to.  The invocation calls the
   function assigned to the action with ACTION_PTR and EVENT_PTR as
   parameters. If ACTION_PTR is null or the actions assigned function is
   null, nothing is done.  EVENT_PTR mustn't be null. */
void
action_invoke (action_t *action_ptr, SDL_Event *event_ptr)
  __attribute__ ((nonnull (2)));

/* A trigger is an abstraction of all button like events such as keyboard
   keys, mouse buttons and joystick buttons, as well as some other events
   such as SDL_Quit since it's common to also map a button for quitting.
   All these triggers share that they generate events when they are
   pressed and released thus it makes sence to unify them to a single
   abstraction, resulting in the enumeration below. */
enum trigger_state_enum
{
  TRIGGER_DOWN,
  TRIGGER_UP,
  TRIGGER_UNKNOWN /* Not a trigger. */
};

typedef enum trigger_state_enum trigger_state_t;

/* Return the trigger state of the event pointed to by EVENT_PTR.  Returns
   TRIGGER_DOWN and TRIGGER_UP on respective button-like events,
   TRIGGER_DOWN on SDL_QUIT, TRIGGER_UNKNOWN for any other type of
   event.  EVENT_PTR mustn't be null. */
trigger_state_t
event_trigger_state (SDL_Event *event)
  __attribute__ ((nonnull));

/* The type of functions that are assigned to plain actions. */
typedef void (*action_plain_function_t) ();

/* Often an action just calls a function with zero arguments when a
   trigger is pressed or released, plain actions gives a convenient
   interface for these situations as they do exactly that. */
struct action_plain_str 
{
  action_t action;
  action_plain_function_t function;
  trigger_state_t trigger_state;
};
  
typedef struct action_plain_str action_plain_t;

/* Initialize the plain action pointed to by PLAIN_ACTION_PTR to call
   FUNCTION on invokation when the invoking event is a trigger with state
   TRIGGER_STATE.  PLAIN_ACTION_PTR mustn't be null.  If FUNCTION is null
   invoking this action does nothing. */  
void
action_plain_init (action_plain_t *plain_action_ptr,
		   action_plain_function_t function,
		   trigger_state_t trigger_state)
  __attribute__ ((nonnull (1)));

#endif /* not ACTION_H */
