/* Display for the the sandbox.
   Copyright (C) 2006 Russell Turner,
                      Thomas Ren Sidor,
                      Carl Fredrik Hammar (hammy.lite@gmail.com)

   This file is part of GP2X Sand.

   GP2X Sand is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   GP2X Sand is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details. 

   You should have received a copy of the GNU General Public License along
   with GP2X Sand; if not, write to the Free Software Foundation, Inc., 51
   Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */

#include "display.h"

#include "SDL.h"
#include <assert.h>

#include "sandbox.h"

SDL_Surface *screen;
int screen_width;
int screen_height;

/* This module is responsible for displaying the contents of the sandbox
   to a surface, most likely the screen surface. */

/* Initialize the sandbox display to use the surface pointed to by
   SURFACE_PTR.  WIDTH and HEIGHT are the width and height of the
   surface.  SURFACE_PTR mustn't be null. */
void
sandbox_display_init (SDL_Surface *surface_ptr, int width, int height)
{
  assert (surface_ptr != 0);
  
  screen = surface_ptr;
  screen_width = width;
  screen_height = height;
}

/* Writes PIXELS to SCREEN's pixels starting at position (X, Y) and the
   number of pixels that packed into PIXELS forward. */
inline
void
screen_raw_set_pixels (Uint16 x, Uint16 y, Uint32 pixels)
{
  Uint32 *ptr = screen->pixels + y * screen->pitch
    + x * screen->format->BytesPerPixel;
  *ptr = pixels;
}

void
screen_pixel_set (vect_t position, Uint16 pixel)
{
  Uint16 *dst = screen->pixels + position.y * screen->pitch
    + position.x * screen->format->BytesPerPixel;
  *dst = pixel;
}

void
screen_rectangle_fill (vect_t position, vect_t dimensions, Uint16 color)
{
  SDL_Rect rect = { position.x, position.y, dimensions.x, dimensions.y };
  SDL_FillRect (screen, &rect, color);
}
  
      
/* Updates the display, drawing the contents of the sandbox to the
   surface assigned to the display. */
void
sandbox_display_update ()
{
  sandbox_sector_t *sectors;
  int sectors_length;
  
  sandbox_sectors_get (&sectors, &sectors_length);
  
  for (int i = 0; i < sectors_length; i++)
    {
      Uint16 dominant_color = sectors[i].dominant_element_ptr->color;
      screen_rectangle_fill (sectors[i].position, sectors[i].dimensions,
			     dominant_color);
      
      if (sectors[i].homogenous_flag && !sectors[i].dirty_flag)
	continue;
      
      /* Position of the sector's top-left corner. */ 
      vect_t start = { sectors[i].position.x, sectors[i].position.y };

      /* Position of the sector's bottom-right corner. */ 
      vect_t end = { start.x + sectors[i].dimensions.x,
		     start.y + sectors[i].dimensions.y };
      
      if (SDL_MUSTLOCK (screen))
	{
	  if (SDL_LockSurface (screen) < 0)
	    return;
	}

      for (vect_t v = start; v.y < end.y; v.y++)
	{
	  for (v.x = start.x; v.x < end.x; v.x++)
	    {
	      particle_t particle = sandbox_particle_get (v);
	      uint16_t color = particle_color_get (particle);
	      
	      if (color != dominant_color)
		screen_pixel_set (v, color);
	    }
	}

      if (SDL_MUSTLOCK(screen))
	  SDL_UnlockSurface(screen);
    }
}

