/* Calculate average frames per second.
   Copyright (C) 2006 Russell Turner,
                      Thomas Ren Sidor,
                      Carl Fredrik Hammar (hammy.lite@gmail.com)

   This file is part of GP2X Sand.

   GP2X Sand is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   GP2X Sand is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details. 

   You should have received a copy of the GNU General Public License along
   with GP2X Sand; if not, write to the Free Software Foundation, Inc., 51
   Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */

#include "fpscounter.h"

/* This module is used to calculate the average fps over the last n
   frames.  Where n is an appropriate number that give an accurate
   average, which is private to this module.  Using it is simply a matter
   of giving it the current time after each frame is displayed. */

#define FPS_COUNTER_FRAMES_N 20 /* Number of frame timings to store. */

struct fps_counter_str
{
  int index; /* Index off frame_timespans to place next time-span. */
  int frame_prev_timestamp;
  int frame_timespans_n; /* Lenght of frame_timespans. */
  int frame_timespans[];
};

typedef struct fps_counter_str fps_counter_t;
  
fps_counter_t fps_counter = { 0, 0, FPS_COUNTER_FRAMES_N,
			      { [0 ... FPS_COUNTER_FRAMES_N] = 0 } };

/* Initialize the fps counter. TIMESTAMP should be the current time in
   milliseconds. */
void
fps_counter_init (int timestamp)
{

  fps_counter.index = 0;
  fps_counter.frame_prev_timestamp = timestamp;
  fps_counter.frame_timespans_n = FPS_COUNTER_FRAMES_N;
  
  for (int i = 0; i < fps_counter.frame_timespans_n; i++)
    fps_counter.frame_timespans[i] = 0;
}

/* Stores the latest frames timestamp for later use to calculate the
   average.  TIMESTAMP should be the time in milliseconds when the frame
   was finished.  Call this function directly after each frame is
   displayed.  If TIMESTAMP is lower than the last timestamp pushed then
   this call will be ignored. */
void
fps_counter_frame_timestamp_push (int timestamp)
{
  int i = fps_counter.index;
  int prev_timestamp = fps_counter.frame_prev_timestamp;

  if (timestamp < prev_timestamp) /* A time-span can't be negative. */
    return;

  fps_counter.frame_timespans[i] = timestamp - prev_timestamp;
  fps_counter.frame_prev_timestamp = timestamp;
  fps_counter.index = (i + 1) % fps_counter.frame_timespans_n;
}
  
/* Calculate the average fps and return it. */
int
fps_counter_framerate_get ()
{
  int total = 0;
  
  for (int i = 0; i < fps_counter.frame_timespans_n; i++)
      total += fps_counter.frame_timespans[i];
  
  int average = total / fps_counter.frame_timespans_n;

  return 1000 / average;
}

